from datetime import datetime
from unittest.mock import Mock

import pytest
from streaming_data_types.run_stop_6s4t import deserialise_6s4t as deserialise_stop

from file_writer_control.JobStatus import JobState, JobStatus
from file_writer_control.WorkerFinder import WorkerFinderBase
from file_writer_control.WriteJob import WriteJob


def test_send_command():
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    data_under_test = b"abcdefghijklmnopqrstwxyz"
    under_test.send_command(data_under_test)
    producer_mock.send.assert_called_once_with(test_topic, data_under_test)


def test_try_start_job():
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    with pytest.raises(NotImplementedError):
        under_test.try_start_job(WriteJob("", "", "", datetime.now()))


def test_try_send_stop_time():
    service_id = "some service_id"
    job_id = "some job id"
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    stop_time = datetime.now()
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    result = under_test.try_send_stop_time(service_id, job_id, stop_time)
    cmd_channel_mock.add_command_id.assert_called_once_with(
        job_id=job_id, command_id=result.command_id
    )
    producer_mock.send.assert_called_once()
    topic_name = producer_mock.send.call_args_list[0].args[0]
    assert topic_name == test_topic
    message = deserialise_stop(producer_mock.send.call_args_list[0].args[1])
    assert message.service_id == service_id
    assert message.job_id == job_id
    assert message.command_id == result.command_id
    assert message.stop_time == int(stop_time.timestamp() * 1000)


def test_try_abort_job_now():
    service_id = "some service_id"
    job_id = "some job id"
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    result = under_test.try_send_abort(service_id, job_id)
    cmd_channel_mock.add_command_id.assert_called_once_with(
        job_id=job_id, command_id=result.command_id
    )
    producer_mock.send.assert_called_once()
    topic_name = producer_mock.send.call_args_list[0].args[0]
    assert topic_name == test_topic
    message = deserialise_stop(producer_mock.send.call_args_list[0].args[1])
    assert message.service_id == service_id
    assert message.job_id == job_id
    assert message.command_id == result.command_id


def test_get_job_status():
    job_id = "some job id"
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    result = under_test.get_job_status(job_id)
    cmd_channel_mock.get_job.assert_called_once_with(job_id)
    assert cmd_channel_mock.get_job.return_value == result


def test_get_job_state_success():
    job_id = "some job id"
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    used_state = JobStatus(job_id)
    used_state.state = JobState.TIMEOUT
    cmd_channel_mock.get_job.return_value = used_state
    result = under_test.get_job_state(job_id)
    cmd_channel_mock.get_job.assert_called_once_with(job_id)
    assert used_state.state == result


def test_get_job_state_failure():
    job_id = "some job id"
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    cmd_channel_mock.get_job.return_value = None
    result = under_test.get_job_state(job_id)
    cmd_channel_mock.get_job.assert_called_once_with(job_id)
    assert JobState.UNAVAILABLE == result


def test_list_commands():
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    assert (
        under_test.list_known_commands() == cmd_channel_mock.list_commands.return_value
    )


def test_list_jobs():
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    assert under_test.list_known_jobs() == cmd_channel_mock.list_jobs.return_value


def test_list_workers():
    cmd_channel_mock = Mock()
    producer_mock = Mock()
    test_topic = "some topic"
    under_test = WorkerFinderBase(test_topic, cmd_channel_mock, producer_mock)
    assert under_test.list_known_workers() == cmd_channel_mock.list_workers.return_value



from copy import copy
from datetime import datetime, timedelta

from streaming_data_types import deserialise_pl72

from file_writer_control.WriteJob import WriteJob


def test_write_generate_job_id():
    under_test = WriteJob("", "", "", datetime.now())
    old_job_id = copy(under_test.job_id)
    under_test.generate_new_job_id()
    assert old_job_id != under_test.job_id


def test_get_start_message():
    under_test = WriteJob("", "", "", datetime.now())
    assert type(under_test.get_start_message()) is bytes


def test_get_start_message_contents():
    structure = "some structure"
    file_name = "some file name"
    start_time = datetime.now()
    stop_time = start_time + timedelta(seconds=10)
    service_id = "some service id"
    broker = "some broker"
    instrument_name = "some instrument name"
    metadata = "some meta data"
    run_name = "some run name"
    under_test = WriteJob(
        nexus_structure=structure,
        file_name=file_name,
        broker=broker,
        start_time=start_time,
        stop_time=stop_time,
        instrument_name=instrument_name,
        run_name=run_name,
        metadata=metadata,
    )
    under_test.service_id = service_id
    message = deserialise_pl72(under_test.get_start_message())
    assert message.nexus_structure == structure
    assert message.filename == file_name
    assert message.service_id == service_id
    assert message.broker == broker
    assert message.instrument_name == instrument_name
    assert message.run_name == run_name
    assert message.start_time == int(start_time.timestamp() * 1000)
    assert message.stop_time == int(stop_time.timestamp() * 1000)
    assert message.job_id == under_test.job_id
    assert message.instrument_name == instrument_name
    assert message.metadata == metadata



import argparse
import os
import signal
import sys
import time
from datetime import datetime, timedelta
from time import time as current_time
from typing import Tuple

from file_writer_control import JobHandler, JobState, WorkerJobPool, WriteJob

JOB_HANDLER: JobHandler
ACK_TIMEOUT: float


def cli_parser() -> argparse.Namespace:
    """
    Parser for the command line interface.
    """

    fw_parser = argparse.ArgumentParser(
        fromfile_prefix_chars="@", description="FileWriter Starter"
    )

    fw_parser.add_argument(
        "-f",
        "--filename",
        metavar="filename",
        type=str,
        required=True,
        help="Name of the output file, e.g., `<filename>.nxs`.",
    )
    fw_parser.add_argument(
        "-j",
        "--job-id",
        metavar="job_id",
        type=str,
        help="The job identifier of the currently running file-writer job. "
        "The job identifier should be a valid UUID.",
    )
    fw_parser.add_argument(
        "-c",
        "--config",
        metavar="json_config",
        type=str,
        required=True,
        help="Path to JSON config file.",
    )
    fw_parser.add_argument(
        "-b",
        "--broker",
        metavar="kafka_broker",
        type=str,
        default="localhost:9092",
        help="Kafka broker port.",
    )
    fw_parser.add_argument(
        "-t",
        "--command-status-topic",
        metavar="consume_topic",
        type=str,
        required=True,
        help="Name of the Kafka topic to listen to" " commands and send status to.",
    )
    fw_parser.add_argument(
        "-p",
        "--job-pool-topic",
        metavar="job_pool_topic",
        type=str,
        required=True,
        help="The Kafka topic that the available file-writers"
        " are listening to for write jobs.",
    )
    fw_parser.add_argument(
        "--timeout",
        metavar="ack_timeout",
        type=float,
        default=5,
        help="How long to wait for timeout on acknowledgement.",
    )
    fw_parser.add_argument(
        "--stop",
        metavar="stop_writing",
        type=float,
        help="How long the file will be written.",
    )

    args = fw_parser.parse_args()

    return args


def file_writer(args: argparse.Namespace) -> None:
    write_job = prepare_write_job(args)
    start_time, timeout = start_write_job(write_job)

    if args.stop:
        stop_write_job(args.stop, start_time, timeout)

    inform_status()


def start_write_job(write_job: WriteJob) -> Tuple[datetime, float]:

    start_handler = JOB_HANDLER.start_job(write_job)
    timeout = int(current_time()) + ACK_TIMEOUT
    start_time = datetime.now()
    while not start_handler.is_done():
        if int(current_time()) > timeout:
            raise ValueError("Timeout.")
    return start_time, timeout


def stop_write_job(stop: float, start_time: datetime, timeout: float) -> None:

    stop_time = start_time + timedelta(seconds=stop)
    stop_handler = JOB_HANDLER.set_stop_time(stop_time)
    while not stop_handler.is_done() and not JOB_HANDLER.is_done():
        if int(current_time()) > timeout:
            raise ValueError("Timeout.")


def stop_write_job_now() -> None:
    if JOB_HANDLER.get_state() == JobState.WRITING:
        JOB_HANDLER.set_stop_time(datetime.now())
        while not JOB_HANDLER.is_done():
            time.sleep(1)
        print("FileWriter successfully stopped.")
    sys.exit()


def prepare_write_job(args: argparse.Namespace) -> WriteJob:
    global JOB_HANDLER
    global ACK_TIMEOUT

    file_name = args.filename
    job_id = args.job_id
    host = args.broker
    command_topic = args.command_status_topic
    pool_topic = args.job_pool_topic
    config = args.config
    ACK_TIMEOUT = args.timeout
    command_channel = WorkerJobPool(f"{host}/{pool_topic}", f"{host}/{command_topic}")
    JOB_HANDLER = JobHandler(worker_finder=command_channel)
    with open(config, "r") as f:
        nexus_structure = f.read()
    if job_id:
        write_job = WriteJob(
            nexus_structure, file_name, host, datetime.now(), job_id=job_id
        )
    else:
        write_job = WriteJob(
            nexus_structure,
            file_name,
            host,
            datetime.now(),
        )
    return write_job


def inform_status() -> None:
    if JOB_HANDLER.get_state() == JobState.WRITING:
        print("Use CTRL-C to enter interactive menu.")
        print("Writing.", end="", flush=True)
    while JOB_HANDLER.get_state() == JobState.WRITING:
        print(".", end="", flush=True)
        time.sleep(1)
        if JOB_HANDLER.get_state() == JobState.DONE:
            print("[DONE]")


def validate_namespace(args: argparse.Namespace) -> None:
    argument_list = [
        args.filename,
        args.config,
        args.broker,
        args.command_status_topic,
        args.job_pool_topic,
    ]
    for arg in argument_list:
        is_empty(arg)

    # Validate extensions for filename and config
    check_file_extension(args.filename, "nxs")
    check_file_extension(args.config, "json")

    # Validate JSON config path
    config_file = args.config
    if not os.path.isfile(config_file):
        raise ValueError(
            "The configuration file " f"`{config_file}` " "does not exist."
        )


def check_file_extension(arg: str, extension: str) -> None:
    if len(arg.split(".")) < 2 or arg.split(".")[1] != extension:
        raise ValueError(
            f"The argument, {arg}, has incorrect extension. "
            "Please use `.nxs` for output file and `.json` for "
            "the configuration."
        )
    is_empty(arg.split(".")[0])


def is_empty(arg: str) -> None:
    if not arg or arg.isspace():
        raise ValueError("A positional argument cannot be an empty string.")


def ask_user_action(signum, frame) -> None:
    user_action = """

    What would you like to do (type 1, 2 or 3 and press Enter)?

    1- Stop FileWriter immediately
    2- Stop FileWriter after a given time (seconds)
    3- Exit CLI without terminating FileWriter

    Or any other key to Continue.

    """

    choice = input(user_action)

    if choice == "1":
        stop_write_job_now()
    elif choice == "2":
        set_time_and_stop()
    elif choice == "3":
        sys.exit()
    else:
        return


def set_time_and_stop() -> None:
    stop_time = input("Stop time in seconds = ")
    try:
        stop_time = float(stop_time)
        timeout = int(current_time()) + ACK_TIMEOUT
        stop_write_job(stop_time, datetime.now(), timeout)
    except ValueError:
        # The CLI will simply continue.
        print("Input should be a float.")


def start_writer():
    # Catch ctrl-c
    signal.signal(signal.SIGINT, ask_user_action)
    # Main
    cli_args = cli_parser()
    validate_namespace(cli_args)
    file_writer(cli_args)


if __name__ == "__main__":
    start_writer()



import argparse
import sys
import time
from datetime import datetime, timedelta
from time import time as current_time

from cli.start_file_writer import is_empty
from file_writer_control import JobHandler, JobState, WorkerJobPool


def cli_parser() -> argparse.Namespace:
    """
    Parser for the command line interface.
    """

    fw_parser = argparse.ArgumentParser(
        fromfile_prefix_chars="@", description="FileWriter Stopper"
    )

    fw_parser.add_argument(
        "-s",
        "--stop",
        metavar="stop",
        type=str,
        help="Stop FileWriter immediately.",
    )
    fw_parser.add_argument(
        "-sa",
        "--stop_after",
        metavar="stop_after",
        nargs=2,
        type=str,
        help="Stop FileWriter after a given time in seconds.",
    )
    fw_parser.add_argument(
        "-b",
        "--broker",
        metavar="kafka_broker",
        type=str,
        default="localhost:9092",
        help="Kafka broker port.",
    )
    fw_parser.add_argument(
        "-t",
        "--command-status-topic",
        metavar="consume_topic",
        type=str,
        required=True,
        help="Name of the Kafka topic to listen to commands and send status to.",
    )
    fw_parser.add_argument(
        "-p",
        "--job-pool-topic",
        metavar="job_pool_topic",
        type=str,
        required=True,
        help="The Kafka topic that the available file-writers"
        " are listening to for write jobs.",
    )
    fw_parser.add_argument(
        "--timeout",
        metavar="ack_timeout",
        type=float,
        default=5,
        help="How long to wait for timeout on acknowledgement.",
    )

    args = fw_parser.parse_args()

    return args


def create_job_handler(args: argparse.Namespace, job_id: str) -> JobHandler:
    host = args.broker
    command_topic = args.command_status_topic
    pool_topic = args.job_pool_topic
    command_channel = WorkerJobPool(f"{host}/{pool_topic}", f"{host}/{command_topic}")
    job_handler = JobHandler(worker_finder=command_channel, job_id=job_id)
    # Required for formation of the handler.
    time.sleep(3)
    return job_handler


def stop_write_job_now(job_handler: JobHandler) -> None:
    if job_handler.get_state() == JobState.WRITING:
        job_handler.set_stop_time(datetime.now())
        while not job_handler.is_done():
            time.sleep(1)
        print("FileWriter successfully stopped.")
    sys.exit()


def stop_write_job(args: argparse.Namespace, job_handler: JobHandler) -> None:
    stop_time = float(args.stop_after[1])
    timeout = int(current_time()) + args.timeout
    stop_time = datetime.now() + timedelta(seconds=stop_time)
    stop_handler = job_handler.set_stop_time(stop_time)
    while not stop_handler.is_done() and not job_handler.is_done():
        if int(current_time()) > timeout:
            raise ValueError("Timeout.")


def verify_write_job(job_handler: JobHandler) -> None:
    if job_handler.get_state() == JobState.WRITING:
        print("The write process is confirmed. Stopping...")
    else:
        raise ValueError(
            "There are no write jobs associated with the "
            "given job id. Please check broker, topic and "
            "id information and try again."
        )


def validate_namespace(args: argparse.Namespace) -> None:
    if args.stop and args.stop_after:
        print(
            "Positional arguments [-s --stop] and [-sa --stop_after] cannot "
            "be used simultaneously."
        )
        sys.exit()
    argument_list = [
        args.stop,
        args.broker,
        args.command_status_topic,
        args.job_pool_topic,
    ]
    for arg in argument_list + args.stop_after:
        if arg:
            is_empty(arg)


def stop_writer():
    cli_args = cli_parser()
    validate_namespace(cli_args)

    _id = cli_args.stop if cli_args.stop else cli_args.stop_after[0]
    handler = create_job_handler(cli_args, _id)
    verify_write_job(handler)

    if cli_args.stop_after:
        stop_write_job(cli_args, handler)
    else:
        stop_write_job_now(handler)


if __name__ == "__main__":
    stop_writer()



import time
from datetime import datetime

from file_writer_control import JobHandler, WorkerJobPool, WriteJob

if __name__ == "__main__":
    kafka_host = "dmsc-kafka01:9092"

    # Job pool is where idle file-writers listen for start messages.
    # The start message can come from any instrument and only one file-writer will
    # respond to it.
    job_pool_topic = "job_pool_topic"

    # Control topic is a "private" topic dedicated to a specific instrument.
    # This is where the selected file-writer writes status messages and
    # listens for stop messages from that instrument.
    control_topic = "ymir_control_topic"

    with open("file_writer_config.json", "r") as f:
        nexus_structure = f.read()

    command_channel = WorkerJobPool(
        f"{kafka_host}/{job_pool_topic}", f"{kafka_host}/{control_topic}"
    )
    job_handler = JobHandler(worker_finder=command_channel)
    start_time = datetime.now()

    write_job = WriteJob(
        nexus_structure,
        "{0:%Y}-{0:%m}-{0:%d}_{0:%H}{0:%M}.nxs".format(start_time),
        kafka_host,
        start_time,
        control_topic=control_topic,
    )

    print("Starting write job")
    start_handler = job_handler.start_job(write_job)
    while not start_handler.is_done():
        time.sleep(1)
    print("Job started")

    input("Hit return to stop writing")

    stop_handler = job_handler.set_stop_time(datetime.now())
    while not stop_handler.is_done():
        time.sleep(1)
    while not job_handler.is_done():
        time.sleep(1)
    print("Job stopped")



import pytest

from file_writer_control.KafkaTopicUrl import KafkaTopicUrl


def test_success_1():
    url_obj = KafkaTopicUrl("kafka://addr:9012/hello_12")
    assert url_obj.host_port == "addr:9012"
    assert url_obj.topic == "hello_12"


def test_success_2():
    url_obj = KafkaTopicUrl("kafka://addr/hello_12")
    assert url_obj.host_port == "addr:9092"
    assert url_obj.topic == "hello_12"


def test_success_3():
    url_obj = KafkaTopicUrl("addr:9012/hello_12")
    assert url_obj.host_port == "addr:9012"
    assert url_obj.topic == "hello_12"


def test_success_4():
    url_obj = KafkaTopicUrl("addr.se:9012/hello_12")
    assert url_obj.host_port == "addr.se:9012"
    assert url_obj.topic == "hello_12"


def test_success_5():
    url_obj = KafkaTopicUrl("192.168.1.21:9012/hello_12")
    assert url_obj.host_port == "192.168.1.21:9012"
    assert url_obj.topic == "hello_12"


def test_failure_1():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafka://:9012/hello_12")


def test_failure_2():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafka://what.com/")


def test_failure_3():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafka://what.com")


def test_failure_4():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafka:/what.com:124/hej")


def test_failure_5():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafkaaa://what.com:124/hej")


def test_failure_6():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("//what.com:124/hej")


def test_failure_7():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafka://what.com:sfa/hej")


def test_failure_8():
    with pytest.raises(RuntimeError):
        KafkaTopicUrl("kafka://what.com:1234/hej/again")



from unittest.mock import Mock

import pytest

from file_writer_control.CommandHandler import CommandHandler
from file_writer_control.CommandStatus import CommandState, CommandStatus


def test_get_state_no_id():
    mock_cmd_ch = Mock()
    mock_cmd_ch.get_command.return_value = None
    cmd_id = "some_command_id"
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    assert under_test.get_state() == CommandState.UNKNOWN
    mock_cmd_ch.get_command.assert_called_once_with(cmd_id)


def test_get_state_with_id():
    mock_cmd_ch = Mock()
    job_id = "some_job_id"
    cmd_id = "some_command_id"
    stand_in_status = CommandStatus(job_id, cmd_id)
    mock_cmd_ch.get_command.return_value = stand_in_status
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    assert under_test.get_state() == stand_in_status.state
    mock_cmd_ch.get_command.assert_called_once_with(cmd_id)


def test_get_error_no_id():
    mock_cmd_ch = Mock()
    mock_cmd_ch.get_command.return_value = None
    cmd_id = "some_command_id"
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    assert under_test.get_message() == ""
    mock_cmd_ch.get_command.assert_called_once_with(cmd_id)


def test_get_error_with_id():
    mock_cmd_ch = Mock()
    job_id = "some_job_id"
    cmd_id = "some_command_id"
    message_string = "some message"
    stand_in_status = CommandStatus(job_id, cmd_id)
    stand_in_status.message = message_string
    mock_cmd_ch.get_command.return_value = stand_in_status
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    assert under_test.get_message() == message_string
    mock_cmd_ch.get_command.assert_called_once_with(cmd_id)


def test_has_timed_out():
    mock_cmd_ch = Mock()
    job_id = "some_job_id"
    cmd_id = "some_command_id"
    stand_in_status = CommandStatus(job_id, cmd_id)
    stand_in_status.state = CommandState.TIMEOUT_RESPONSE
    mock_cmd_ch.get_command.return_value = stand_in_status
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    with pytest.raises(RuntimeError):
        under_test.is_done()
    mock_cmd_ch.get_command.assert_called_once_with(cmd_id)


def test_is_not_done():
    mock_cmd_ch = Mock()
    job_id = "some_job_id"
    cmd_id = "some_command_id"
    stand_in_status = CommandStatus(job_id, cmd_id)
    stand_in_status.state = CommandState.ERROR
    mock_cmd_ch.get_command.return_value = stand_in_status
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    with pytest.raises(RuntimeError):
        under_test.is_done()
    assert mock_cmd_ch.get_command.call_count >= 1


def test_is_done():
    mock_cmd_ch = Mock()
    job_id = "some_job_id"
    cmd_id = "some_command_id"
    stand_in_status = CommandStatus(job_id, cmd_id)
    stand_in_status.state = CommandState.SUCCESS
    mock_cmd_ch.get_command.return_value = stand_in_status
    under_test = CommandHandler(mock_cmd_ch, cmd_id)
    assert under_test.is_done()
    mock_cmd_ch.get_command.assert_called_once_with(cmd_id)



import pytest

from file_writer_control.CommandStatus import (
    CommandState,
    CommandStatus,
    COMMAND_STATUS_TIMEOUT,
)
from datetime import timedelta


def test_init():
    job_id = "some job id"
    command_id = "some command id"
    under_test1 = CommandStatus(job_id=job_id, command_id=command_id)
    assert under_test1.job_id == job_id
    assert under_test1.command_id == command_id
    assert under_test1.state == CommandState.NO_COMMAND


def test_eq_exception():
    under_test1 = CommandStatus("job_id", "command_id")
    with pytest.raises(NotImplementedError):
        under_test1 == 1


def test_eq_success():
    under_test1 = CommandStatus("job_id", "command_id")
    under_test2 = CommandStatus("job_id", "command_id")
    assert under_test1 == under_test2


def test_eq_failure_1():
    under_test1 = CommandStatus("job_id1", "command_id1")
    under_test2 = CommandStatus("job_id2", "command_id1")
    assert under_test1 != under_test2


def test_eq_failure_2():
    under_test1 = CommandStatus("job_id1", "command_id1")
    under_test2 = CommandStatus("job_id1", "command_id2")
    assert under_test1 != under_test2


def test_update_failure():
    under_test1 = CommandStatus("job_id1", "command_id1")
    under_test2 = CommandStatus("job_id1", "command_id2")
    with pytest.raises(RuntimeError):
        under_test2.update_status(under_test1)


def test_update_success():
    under_test1 = CommandStatus("job_id1", "command_id1")
    under_test2 = CommandStatus("job_id1", "command_id1")
    under_test2.update_status(under_test1)  # No exception


def test_timeout1():
    under_test = CommandStatus("job_id1", "command_id1")
    assert under_test.timeout == COMMAND_STATUS_TIMEOUT


def test_timeout2():
    test_time = timedelta(minutes=2)
    under_test = CommandStatus("job_id1", "command_id1", test_time)
    assert under_test.timeout == test_time
    test_time2 = timedelta(seconds=30)
    under_test.timeout = test_time2
    assert under_test.timeout == test_time2


def test_update_message():
    under_test1 = CommandStatus("job_id1", "command_id1")
    under_test2 = CommandStatus("job_id1", "command_id1")
    test_msg = "some test message"
    under_test1.message = test_msg
    assert under_test2.message == ""
    under_test2.update_status(under_test1)  # No exception
    assert under_test2.message == test_msg


def test_set_message():
    under_test1 = CommandStatus("job_id1", "command_id1")
    last_update = under_test1.last_update
    test_msg = "some test message"
    assert under_test1.message == ""
    under_test1.message = test_msg
    assert under_test1.message == test_msg
    assert under_test1.last_update != last_update


def test_set_state():
    under_test1 = CommandStatus("job_id1", "command_id1")
    last_update = under_test1.last_update
    test_state = CommandState.SUCCESS
    assert under_test1.state == CommandState.NO_COMMAND
    under_test1.state = test_state
    assert under_test1.state == test_state
    assert under_test1.last_update != last_update



import pytest

from file_writer_control.WorkerStatus import WorkerState, WorkerStatus


def test_init():
    test_id = "some_id"
    under_test1 = WorkerStatus(test_id)
    assert test_id == under_test1.service_id
    assert under_test1.state == WorkerState.UNAVAILABLE


def test_eq_wrong_type():
    under_test1 = WorkerStatus("some_id")
    with pytest.raises(NotImplementedError):
        under_test1 == 1


def test_eq_true():
    under_test1 = WorkerStatus("some_id")
    under_test2 = WorkerStatus("some_id")
    assert under_test1 == under_test2


def test_eq_false():
    under_test1 = WorkerStatus("some_id")
    under_test2 = WorkerStatus("some_id2")
    assert under_test1 != under_test2


def test_update_status_wrong_id():
    under_test1 = WorkerStatus("some_id")
    under_test2 = WorkerStatus("some_id2")
    with pytest.raises(RuntimeError):
        under_test1.update_status(under_test2)


def test_update_status_ok():
    test_state = WorkerState.WRITING
    under_test1 = WorkerStatus("some_id")
    under_test1.state = WorkerState.IDLE
    under_test2 = WorkerStatus("some_id")
    under_test2.state = test_state
    last_update = under_test1.last_update
    under_test1.update_status(under_test2)  # No throw
    assert last_update != under_test1.last_update
    assert under_test1.state == test_state


def test_state_change_date_change():
    under_test1 = WorkerStatus("some_id")
    last_update = under_test1.last_update
    under_test1.state = WorkerState.WRITING
    assert last_update != under_test1.last_update



from datetime import datetime, timedelta

from file_writer_control.CommandChannel import CommandChannel
from file_writer_control.CommandStatus import CommandState, CommandStatus
from file_writer_control.InThreadStatusTracker import DEAD_ENTITY_TIME_LIMIT
from file_writer_control.JobStatus import JobState, JobStatus
from file_writer_control.WorkerStatus import WorkerState, WorkerStatus


def test_add_job_id():
    under_test = CommandChannel("localhost:42/some_topic")
    assert len(under_test.list_jobs()) == 0
    job_id_1 = "some_job_id_1"
    assert under_test.get_job(job_id_1) is None
    under_test.add_job_id(job_id_1)
    assert len(under_test.list_jobs()) == 1
    assert under_test.get_job(job_id_1).job_id == job_id_1
    job_id_2 = "some_job_id_2"
    assert under_test.get_job(job_id_2) is None
    under_test.add_job_id(job_id_2)
    assert len(under_test.list_jobs()) == 2
    assert under_test.get_job(job_id_2).job_id == job_id_2
    under_test.stop_thread()


def test_timed_out():
    under_test = CommandChannel("localhost:42/some_topic")
    now = datetime.now()
    test_worker = WorkerStatus("some_service_id")
    test_worker.state = WorkerState.UNKNOWN
    under_test.map_of_workers["some_id"] = test_worker

    test_job = JobStatus("some_id")
    test_job.state = JobState.WAITING
    under_test.map_of_jobs["some_id"] = test_job

    test_command = CommandStatus("some_id", "some_other_id")
    test_command.state = CommandState.WAITING_RESPONSE
    under_test.map_of_commands["some_id"] = test_command

    time_diff = timedelta(minutes=5)
    under_test.update_workers(now + time_diff)
    assert under_test.map_of_commands["some_id"].state == CommandState.TIMEOUT_RESPONSE
    assert under_test.map_of_jobs["some_id"].state == JobState.TIMEOUT
    assert under_test.map_of_workers["some_id"].state == WorkerState.UNAVAILABLE


def test_prune_old():
    under_test = CommandChannel("localhost:42/some_topic")
    now = datetime.now()
    test_worker = WorkerStatus("some_service_id")
    test_worker.state = WorkerState.IDLE
    under_test.map_of_workers["some_id"] = test_worker

    test_job = JobStatus("some_id")
    test_job.state = JobState.WRITING
    under_test.map_of_jobs["some_id"] = test_job

    test_command = CommandStatus("some_id", "some_other_id")
    test_command.state = CommandState.SUCCESS
    under_test.map_of_commands["some_id"] = test_command

    under_test.update_workers(now + timedelta(seconds=60))

    time_diff = timedelta(minutes=5)
    under_test.update_workers(now + DEAD_ENTITY_TIME_LIMIT - time_diff)
    assert len(under_test.map_of_commands) == 1
    assert len(under_test.map_of_jobs) == 1
    assert len(under_test.map_of_workers) == 1
    under_test.update_workers(now + DEAD_ENTITY_TIME_LIMIT + time_diff)
    assert len(under_test.map_of_commands) == 0
    assert len(under_test.map_of_jobs) == 0
    assert len(under_test.map_of_workers) == 0


def test_add_command_id():
    under_test = CommandChannel("localhost:42/some_topic")
    assert len(under_test.list_commands()) == 0
    command_id_1 = "some_command_id_1"
    job_id = "some_job_id"
    assert under_test.get_command(command_id_1) is None
    under_test.add_command_id(job_id, command_id_1)
    assert len(under_test.list_commands()) == 1
    assert under_test.get_command(command_id_1).command_id == command_id_1
    command_id_2 = "some_command_id_2"
    assert under_test.get_command(command_id_2) is None
    under_test.add_command_id(job_id, command_id_2)
    assert len(under_test.list_commands()) == 2
    assert under_test.get_command(command_id_2).command_id == command_id_2
    under_test.stop_thread()


def test_update_worker_status():
    under_test = CommandChannel("localhost:42/some_topic")
    service_id = "some_id"
    new_status = WorkerStatus(service_id)
    assert len(under_test.list_workers()) == 0
    assert under_test.get_worker(service_id) is None
    under_test.status_queue.put(new_status)
    under_test.update_workers()
    assert len(under_test.list_workers()) == 1
    assert under_test.get_worker(service_id).service_id == service_id
    under_test.stop_thread()


def test_update_job_status():
    under_test = CommandChannel("localhost:42/some_topic")
    job_id = "some_id"
    new_status = JobStatus(job_id)
    assert len(under_test.list_jobs()) == 0
    assert under_test.get_job(job_id) is None
    under_test.status_queue.put(new_status)
    under_test.update_workers()
    assert len(under_test.list_jobs()) == 1
    assert under_test.get_job(job_id).job_id == job_id
    under_test.stop_thread()


def test_update_command_status():
    under_test = CommandChannel("localhost:42/some_topic")
    job_id = "some_other_id"
    command_id = "some_id"
    new_status = CommandStatus(job_id, command_id)
    assert len(under_test.list_commands()) == 0
    assert under_test.get_command(command_id) is None
    under_test.status_queue.put(new_status)
    under_test.update_workers()
    assert len(under_test.list_commands()) == 1
    assert under_test.get_command(command_id).command_id == command_id
    under_test.stop_thread()
    del under_test


def test_stop_thread_twice():
    under_test = CommandChannel("localhost:42/some_topic")
    under_test.stop_thread()
    under_test.stop_thread()  # No exception



import time

from file_writer_control import WorkerJobPool


def print_current_state(channel: WorkerJobPool):
    print("Known workers")
    w_format = "{:45s}{:30s}"
    print(w_format.format("Service id", "Current state"))
    print("-" * 80)
    for worker in channel.list_known_workers():
        print(w_format.format(worker.service_id, worker.state))

    print("\nKnown jobs")
    j_format = "{:26s}{:30s}{:30s}"
    print(j_format.format("Service id", "Job id", "Current state"))
    print("-" * 80)
    for job in channel.list_known_jobs():
        print(j_format.format(job.service_id, job.job_id, job.state))
        if job.file_name is not None:
            print(f"    File name: {job.file_name}")
        if len(job.message) > 0:
            print(f"    Message: {job.message}")

    print("\nKnown commands")
    c_format = j_format
    print(c_format.format("Job id", "Command id", "Current state"))
    print("-" * 80)
    for command in channel.list_known_commands():
        print(c_format.format(command.job_id, command.command_id, command.state))
        if len(command.message) > 0:
            print("    Message: {}".format(command.message))


if __name__ == "__main__":
    kafka_host = "dmsc-kafka01:9092"
    worker_job_pool = WorkerJobPool(
        f"{kafka_host}/job_pool_topic", f"{kafka_host}/command_topic"
    )
    time.sleep(10)
    print_current_state(worker_job_pool)



import time
from datetime import datetime, timedelta

from file_writer_control import JobHandler
from file_writer_control import WorkerJobPool
from file_writer_control import WriteJob

if __name__ == "__main__":
    kafka_host = "dmsc-kafka01:9092"
    worker_job_pool = WorkerJobPool(
        "{}/job_topic".format(kafka_host), "{}/command_topic".format(kafka_host)
    )
    job_handler = JobHandler(worker_finder=worker_job_pool)
    start_time = datetime.now()
    with open("file_writer_config.json", "r") as f:
        nexus_structure = f.read()
    write_job = WriteJob(
        nexus_structure,
        "{0:%Y}-{0:%m}-{0:%d}_{0:%H}{0:%M}.nxs".format(start_time),
        kafka_host,
        start_time,
    )

    print("Starting write job")
    start_handler = job_handler.start_job(write_job)
    while not start_handler.is_done():
        time.sleep(1)
    stop_time = start_time + timedelta(seconds=60)
    stop_handler = job_handler.set_stop_time(stop_time)
    while not stop_handler.is_done():
        time.sleep(1)
    while not job_handler.is_done():
        time.sleep(1)
    print("Write job is done")



import pytest

from file_writer_control.JobStatus import JobState, JobStatus


def test_update_status_exception():
    under_test1 = JobStatus("Some_id1")
    under_test2 = JobStatus("Some_id2")
    with pytest.raises(RuntimeError):
        under_test1.update_status(under_test2)


def test_update_status_success():
    under_test1 = JobStatus("Some_id1")
    assert under_test1.message == ""
    test_message = "some test message"
    under_test2 = JobStatus("Some_id1")
    under_test2.message = test_message
    under_test1.update_status(under_test2)  # No exception
    assert under_test1.message == test_message


def test_set_service_id():
    under_test1 = JobStatus("Some_id1")
    assert under_test1.service_id == ""
    test_service_id = "some service id"
    old_last_update = under_test1.last_update
    under_test1.service_id = test_service_id
    assert old_last_update != under_test1.last_update
    assert under_test1.service_id == test_service_id
    with pytest.raises(RuntimeError):
        under_test1.service_id = "some other service id"


def test_set_message():
    under_test1 = JobStatus("Some_id1")
    test_message = "some message"
    assert under_test1.message == ""
    last_update = under_test1.last_update
    under_test1.message = test_message
    assert under_test1.message == test_message
    assert last_update != under_test1.last_update


def test_set_state():
    under_test1 = JobStatus("Some_id1")
    test_state = JobState.ERROR
    last_update = under_test1.last_update
    assert under_test1.state == JobState.WAITING
    under_test1.state = test_state
    assert under_test1.state == test_state
    assert last_update != under_test1.last_update



from datetime import datetime

import pytest
from streaming_data_types.action_response_answ import (
    ActionOutcome,
    ActionType,
    Response,
)
from streaming_data_types.status_x5f2 import StatusMessage

from file_writer_control.CommandStatus import CommandState
from file_writer_control.JobStatus import JobState
from file_writer_control.StateExtractor import (
    extract_job_state_from_answer,
    extract_state_from_command_answer,
    extract_worker_state_from_status,
)
from file_writer_control.WorkerStatus import WorkerState


@pytest.mark.parametrize(
    "status_input,state",
    [
        ('{"state":"writing"}', WorkerState.WRITING),
        ('{"state":"idle"}', WorkerState.IDLE),
        ('{"state":"some_state"}', WorkerState.UNKNOWN),
    ],
)
def test_get_worker_state(status_input, state):
    message = StatusMessage(
        "name",
        "version",
        "service_id",
        "host_name",
        "process_id",
        update_interval=5,
        status_json=status_input,
    )
    assert extract_worker_state_from_status(message) == state


@pytest.mark.parametrize(
    "status_input,state",
    [
        (ActionOutcome.Success, CommandState.SUCCESS),
        (ActionOutcome.Failure, CommandState.ERROR),
        (12, CommandState.ERROR),
    ],
)
def test_get_state_from_command_answer(status_input, state):
    answer = Response(
        "service_id",
        "job_id",
        "command_id",
        ActionType.SetStopTime,
        status_input,
        "some message",
        1234,
        datetime.now(),
    )
    assert extract_state_from_command_answer(answer) == state


@pytest.mark.parametrize(
    "action,outcome,state",
    [
        (ActionType.SetStopTime, ActionOutcome.Success, None),
        (ActionType.StartJob, ActionOutcome.Success, JobState.WRITING),
        (ActionType.SetStopTime, ActionOutcome.Failure, None),
        (ActionType.StartJob, ActionOutcome.Failure, JobState.ERROR),
    ],
)
def test_get_job_state_from_answer_done(action, outcome, state):
    answer = Response(
        "service_id",
        "job_id",
        "command_id",
        action,
        outcome,
        "some message",
        1234,
        datetime.now(),
    )
    assert extract_job_state_from_answer(answer) is state



from datetime import datetime, timedelta
from queue import Queue
from unittest.mock import Mock

from streaming_data_types import deserialise_6s4t as deserialise_stop
from streaming_data_types import deserialise_answ as deserialise_answer
from streaming_data_types import deserialise_pl72 as deserialise_start
from streaming_data_types import deserialise_wrdn as deserialise_done
from streaming_data_types import deserialise_x5f2 as deserialise_status
from streaming_data_types import serialise_6s4t as serialise_stop
from streaming_data_types import serialise_answ as serialise_answer
from streaming_data_types import serialise_pl72 as serialise_start
from streaming_data_types import serialise_wrdn as serialise_done
from streaming_data_types import serialise_x5f2 as serialise_status
from streaming_data_types.action_response_answ import (
    ActionOutcome,
    ActionType,
    Response,
)
from streaming_data_types.finished_writing_wrdn import WritingFinished
from streaming_data_types.run_start_pl72 import RunStartInfo
from streaming_data_types.run_stop_6s4t import RunStopInfo
from streaming_data_types.status_x5f2 import StatusMessage

from file_writer_control.CommandStatus import CommandState, CommandStatus
from file_writer_control.InThreadStatusTracker import (
    DEAD_ENTITY_TIME_LIMIT,
    InThreadStatusTracker,
)
from file_writer_control.JobStatus import JobState, JobStatus
from file_writer_control.WorkerStatus import WorkerState, WorkerStatus


def test_process_status_once():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    status_update = StatusMessage(
        "name",
        "version",
        "service_id",
        "host_name",
        "process_id",
        update_interval=5,
        status_json='{"state":"writing","job_id":"some_job_id","file_being_written":"some_file_name.nxs"}',
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_status(status_update)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    assert type(status_queue.get()) is WorkerStatus
    assert len(under_test.known_jobs) == 1
    keys = under_test.known_jobs.keys()
    print(list(keys)[0])
    assert under_test.known_jobs[list(keys)[0]].file_name == "some_file_name.nxs"
    assert under_test.known_jobs[list(keys)[0]].state == JobState.WRITING
    assert under_test.known_jobs[list(keys)[0]].metadata == {
        "state": "writing",
        "job_id": "some_job_id",
        "file_being_written": "some_file_name.nxs",
    }


def test_process_status_twice_two_updates_1():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    status_update = StatusMessage(
        "name",
        "version",
        "service_id",
        "host_name",
        "process_id",
        update_interval=5,
        status_json='{"state":"writing","job_id":"some_job_id","file_being_written":"some_file_name.nxs"}',
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_status(status_update)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    assert type(status_queue.get()) is WorkerStatus
    now = datetime.now()
    under_test.process_status(status_update)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()


def test_process_status_twice_two_updates_2():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    status_update = StatusMessage(
        "name",
        "version",
        "service_id",
        "host_name",
        "process_id",
        update_interval=5,
        status_json='{"state":"writing","job_id":"some_job_id","file_being_written":"some_file_name.nxs"}',
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_status(status_update)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    assert type(status_queue.get()) is WorkerStatus
    status_update = StatusMessage(
        "name",
        "version",
        "service_id",
        "host_name",
        "process_id",
        update_interval=5,
        status_json='{"state":"idle"}',
    )
    now = datetime.now()
    under_test.process_status(status_update)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()


def test_process_answer_set_stop_time():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    answer = Response(
        "service_id",
        "job_id",
        "command_id",
        ActionType.SetStopTime,
        ActionOutcome.Success,
        "some message",
        0,
        datetime.now(),
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_answer(answer)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    assert type(status_queue.get()) is WorkerStatus
    assert type(status_queue.get()) is JobStatus
    assert type(status_queue.get()) is CommandStatus
    assert status_queue.empty()


def test_process_answer_set_stop_time_twice():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    answer = Response(
        "service_id",
        "job_id",
        "command_id",
        ActionType.SetStopTime,
        ActionOutcome.Success,
        "some message",
        0,
        datetime.now(),
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_answer(answer)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    assert type(status_queue.get()) is WorkerStatus
    assert type(status_queue.get()) is JobStatus
    assert type(status_queue.get()) is CommandStatus
    now = datetime.now()
    under_test.process_answer(answer)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    assert type(status_queue.get()) is JobStatus
    assert type(status_queue.get()) is CommandStatus
    assert status_queue.empty()


def test_prune_old():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    now = datetime.now()
    test_worker = WorkerStatus("some_service_id")
    test_worker.state = WorkerState.IDLE
    under_test.known_workers["some_id"] = test_worker

    test_job = JobStatus("some_id")
    test_job.state = JobState.WRITING
    under_test.known_jobs["some_id"] = test_job

    test_command = CommandStatus("some_id", "some_other_id")
    test_command.state = CommandState.SUCCESS
    under_test.known_commands["some_id"] = test_command

    time_diff = timedelta(minutes=5)
    under_test.prune_dead_entities(now + DEAD_ENTITY_TIME_LIMIT - time_diff)
    assert len(under_test.known_commands) == 1
    assert len(under_test.known_jobs) == 1
    assert len(under_test.known_workers) == 1
    under_test.prune_dead_entities(now + DEAD_ENTITY_TIME_LIMIT + time_diff)
    assert len(under_test.known_commands) == 0
    assert len(under_test.known_jobs) == 0
    assert len(under_test.known_workers) == 0


def test_process_answer_start_job():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    answer = Response(
        "service_id",
        "job_id",
        "job_id",
        ActionType.StartJob,
        ActionOutcome.Success,
        "some message",
        0,
        datetime.now(),
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_answer(answer)
    under_test.send_status_if_updated(now)
    assert type(status_queue.get()) is WorkerStatus
    assert type(status_queue.get()) is JobStatus
    assert type(status_queue.get()) is CommandStatus
    assert status_queue.empty()


def test_process_msg_start():
    start_msg = serialise_start("job id", "file name")
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    under_test.process_start = Mock()
    under_test.process_message(start_msg)
    under_test.process_start.assert_called_once_with(deserialise_start(start_msg))


def test_process_msg_stop():
    stop_msg = serialise_stop("job id")
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    under_test.process_set_stop_time = Mock()
    under_test.process_message(stop_msg)
    under_test.process_set_stop_time.assert_called_once_with(deserialise_stop(stop_msg))


def test_process_msg_answer():
    answ_msg = serialise_answer(
        "service id",
        "job id",
        "command id",
        ActionType.StartJob,
        ActionOutcome.Success,
        "some message",
        0,
        datetime.now(),
    )
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    under_test.process_answer = Mock()
    under_test.process_message(answ_msg)
    under_test.process_answer.assert_called_once_with(deserialise_answer(answ_msg))


def test_process_msg_has_stopped():
    stopped_msg = serialise_done("service id", "job id", True, "file name")
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    under_test.process_stopped = Mock()
    under_test.process_message(stopped_msg)
    under_test.process_stopped.assert_called_once_with(deserialise_done(stopped_msg))


def test_process_msg_status():
    status_msg = serialise_status(
        "sw", "v1", "service_id", "host", 12, 142, status_json="{}"
    )
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    under_test.process_status = Mock()
    under_test.process_message(status_msg)
    under_test.process_status.assert_called_once_with(deserialise_status(status_msg))


def test_process_stopped_ok():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    job_id = "some_job_id"
    service_id = "some_service_id"
    stopped = WritingFinished(
        service_id,
        job_id,
        False,
        "FileName",
        """{"key": "meta data"}""",
        "some message",
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_stopped(stopped)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    worker_status = status_queue.get()
    job_status = status_queue.get()
    assert status_queue.empty()
    assert worker_status.state == WorkerState.IDLE
    assert job_status.state == JobState.DONE
    assert job_status.metadata == {"key": "meta data"}


def test_process_stopped_error():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    job_id = "some_job_id"
    service_id = "some_service_id"
    stopped = WritingFinished(
        service_id, job_id, True, "FileName", """{"key": "meta data"}""", "some message"
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_stopped(stopped)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    worker_status = status_queue.get()
    job_status = status_queue.get()
    assert status_queue.empty()
    assert worker_status.state == WorkerState.IDLE
    assert job_status.state == JobState.ERROR
    assert job_status.metadata is None


def test_process_start():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    job_id = "some_job_id"
    service_id = "some_service_id"
    start = RunStartInfo(
        job_id=job_id,
        filename="file name",
        start_time=datetime.now(),
        stop_time=datetime.now(),
        run_name="run_name",
        nexus_structure="nxs structure",
        service_id=service_id,
        instrument_name="instrument name",
        broker="broker",
        metadata="{}",
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_start(start)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    job_status = status_queue.get()
    command_status = status_queue.get()
    assert status_queue.empty()
    assert command_status.state == CommandState.WAITING_RESPONSE
    assert job_status.state == JobState.WAITING


def test_process_set_stop_time():
    status_queue = Queue()
    under_test = InThreadStatusTracker(status_queue)
    job_id = "some_job_id"
    service_id = "some_service_id"
    command_id = "some command id"
    stop = RunStopInfo(
        job_id=job_id,
        stop_time=datetime.now(),
        run_name="run_name",
        service_id=service_id,
        command_id=command_id,
    )
    assert status_queue.empty()
    now = datetime.now()
    under_test.process_set_stop_time(stop)
    under_test.send_status_if_updated(now)
    assert not status_queue.empty()
    command_status = status_queue.get()
    assert status_queue.empty()
    assert command_status.state == CommandState.WAITING_RESPONSE



import time
from datetime import datetime, timedelta

from file_writer_control import CommandState
from file_writer_control import JobHandler
from file_writer_control import JobState
from file_writer_control import WorkerJobPool
from file_writer_control import WriteJob

if __name__ == "__main__":
    kafka_host = "dmsc-kafka01:9092"
    worker_job_pool = WorkerJobPool(
        "{}/job_topic".format(kafka_host), "{}/command_topic".format(kafka_host)
    )
    job_handler = JobHandler(worker_finder=worker_job_pool)
    start_time = datetime.now()
    with open("file_writer_config.json", "r") as f:
        nexus_structure = f.read()
    write_job = WriteJob(
        nexus_structure,
        "{0:%Y}-{0:%m}-{0:%d}_{0:%H}{0:%M}.nxs".format(start_time),
        kafka_host,
        start_time,
    )

    print("Starting write job")
    start_handler = job_handler.start_job(write_job)
    while not start_handler.is_done():
        print("Waiting: {}".format(start_handler.get_state()))
        if start_handler.get_state() == CommandState.ERROR:
            print(
                "Got error when starting job. The error was: {}".format(
                    start_handler.get_message()
                )
            )
            exit(-1)
        elif start_handler.get_state() == CommandState.TIMEOUT_RESPONSE:
            print("Failed to start write job due to start command timeout.")
            exit(-1)
        time.sleep(1)
    print("Write job started")
    stop_time = start_time + timedelta(seconds=60)
    print("Setting stop time")
    stop_handler = job_handler.set_stop_time(stop_time)

    while not stop_handler.is_done():
        time.sleep(1)
        if stop_handler.get_state() == CommandState.ERROR:
            print(
                f"Got error when starting job. The error was: {start_handler.get_message()}"
            )
            exit(-1)
        elif stop_handler.get_state() == CommandState.TIMEOUT_RESPONSE:
            print("Failed to set stop time for write job due to timeout.")
            exit(-1)
    print("Stop time has been set")
    print("Waiting for write job to finish")
    while not job_handler.is_done():
        if job_handler.get_state() == JobState.ERROR:
            print(f"Got job error. The error was: {job_handler.get_message()}")
            exit(-1)
        time.sleep(1)
    print("Write job finished")



from datetime import datetime
from unittest.mock import Mock

import pytest

from file_writer_control.JobHandler import JobHandler
from file_writer_control.JobStatus import JobState, JobStatus
from file_writer_control.WriteJob import WriteJob


def test_default_members():
    worker_finder_mock = Mock()
    under_test = JobHandler(worker_finder_mock)
    assert under_test.job_id == ""
    assert under_test.worker_finder is worker_finder_mock


def test_start_job():
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock = Mock()
    under_test = JobHandler(worker_finder_mock)
    assert (
        under_test.start_job(test_job) == worker_finder_mock.try_start_job.return_value
    )
    worker_finder_mock.try_start_job.assert_called_once_with(test_job)


def test_get_state_no_job_id():
    worker_finder_mock = Mock()
    under_test = JobHandler(worker_finder_mock)
    assert under_test.get_state() == worker_finder_mock.get_job_state.return_value
    worker_finder_mock.get_job_state.assert_called_once_with("")


def test_error_string_no_id():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock.get_job_status.return_value = None
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    assert under_test.get_message() == ""
    worker_finder_mock.get_job_status.assert_called_once_with(test_job.job_id)


def test_error_string_with_id():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    test_job_status = JobStatus(test_job.job_id)
    test_job_status.message = "test msg"
    worker_finder_mock.get_job_status.return_value = test_job_status
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    assert under_test.get_message() is test_job_status.message
    worker_finder_mock.get_job_status.assert_called_once_with(test_job.job_id)


def test_set_stop_time_no_id():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock.get_job_status.return_value = None
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    test_stop_time = datetime.now()
    assert under_test.set_stop_time(test_stop_time) is None


def test_set_stop_time_with_id():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    test_job_status = JobStatus(test_job.job_id)
    test_job_status.service_id = "some_service_id"
    worker_finder_mock.get_job_status.return_value = test_job_status
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    test_stop_time = datetime.now()
    assert (
        under_test.set_stop_time(test_stop_time)
        is worker_finder_mock.try_send_stop_time.return_value
    )
    worker_finder_mock.try_send_stop_time.assert_called_once_with(
        test_job_status.service_id, test_job.job_id, test_stop_time
    )


def test_abort_write_job_no_id():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock.get_job_status.return_value = None
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    assert under_test.abort_write_job() is None


def test_abort_write_job_with_id():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    test_job_status = JobStatus(test_job.job_id)
    test_job_status.service_id = "some_service_id"
    worker_finder_mock.get_job_status.return_value = test_job_status
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    assert (
        under_test.abort_write_job() is worker_finder_mock.try_send_abort.return_value
    )
    worker_finder_mock.try_send_abort.assert_called_once_with(
        test_job_status.service_id, test_job.job_id
    )


def test_is_not_done():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock.get_job_state.return_value = JobState.ERROR
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    with pytest.raises(RuntimeError):
        under_test.is_done()
    assert worker_finder_mock.get_job_state.call_count >= 1


def test_timed_out():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock.get_job_state.return_value = JobState.TIMEOUT
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    with pytest.raises(RuntimeError):
        under_test.is_done()
    assert worker_finder_mock.get_job_state.call_count >= 1


def test_is_done():
    worker_finder_mock = Mock()
    test_job = WriteJob("{}", "some_file_name", "some_broker", datetime.now())
    worker_finder_mock.get_job_state.return_value = JobState.DONE
    under_test = JobHandler(worker_finder_mock)
    under_test.start_job(test_job)
    assert under_test.is_done()
    worker_finder_mock.get_job_state.assert_called_once_with(test_job.job_id)
